/**********************************************************************
* file:   mpltd.c
* Author: Massimiliano piccinini
*
* view "Packet Capture With libpcap and other Low Level Network Tricks"
* at http://www.cet.nau.edu/~mc8/Socket/Tutorials/section1.html 
* for information on packet sniffing with libpcap.
*
* Version 0.8.0
**********************************************************************/
#include "mpltd.h"

#include <pcap.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <signal.h>
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netinet/if_ether.h> 
#include <net/ethernet.h>
#include <netinet/ether.h>
#include <netinet/ip.h>
#include <sys/time.h>
#include <sys/syslog.h>

#ifndef MIN
#define MIN(a,b) (((a) < (b)) ? (a) : (b))
#endif

// Structure for definition of 1 counter. 1 counter count in and out bytes
typedef struct {
char	*name;
long	in;
long	out;
} t_counter;

// Variable that manage the counters
t_counter *Counters = NULL; 
long	   nCounters=0;
long	   aCounters=0;
long	   idxUnClassified=0;

// Variables to simplify the assigment of traffic to counters
// For every port they have the idx of the counter
unsigned short transcode_UDP[65536];
unsigned short transcode_cliTCP[65536];
unsigned short transcode_srvTCP[65536];
#define NOCOUNTER USHRT_MAX 

// Constant to define mask of type of traffic
#define	TRAF_TCP 1
#define	TRAF_UDP 2
#define	TRAF_CLI 4
#define	TRAF_SRV 8

// My version of ip - udp - tcp header structure 
    
struct my_ip 
{
	u_int8_t	ip_vhl;		/* header length, version */
#define IP_V(ip)	(((ip)->ip_vhl & 0xf0) >> 4)
#define IP_HL(ip)	((ip)->ip_vhl & 0x0f)
	u_int8_t	ip_tos;		/* type of service */
	u_int16_t	ip_len;		/* total length */
	u_int16_t	ip_id;		/* identification */
	u_int16_t	ip_off;		/* fragment offset field */
#define	IP_DF 0x4000			/* dont fragment flag */
#define	IP_MF 0x2000			/* more fragments flag */
#define	IP_OFFMASK 0x1fff		/* mask for fragmenting bits */
	u_int8_t	ip_ttl;		/* time to live */
	u_int8_t	ip_p;		/* protocol */
	u_int16_t	ip_sum;		/* checksum */
	struct	in_addr ip_src,ip_dst;	/* source and dest address */

	// La parte seguente ha senso solo per TCP e UDP
	u_int16_t	p_source;
	u_int16_t	p_dest;

	// La parte seguente ha senso solo per TCP
    u_int32_t tcp_seq;
    u_int32_t tcp_seqack;
#  if __BYTE_ORDER == __LITTLE_ENDIAN
    u_int16_t tcp_res1:4;
    u_int16_t tcp_doff:4;
    u_int16_t tcp_fin:1;
    u_int16_t tcp_syn:1;
    u_int16_t tcp_rst:1;
    u_int16_t tcp_psh:1;
    u_int16_t tcp_ack:1;
    u_int16_t tcp_urg:1;
    u_int16_t tcp_res2:2;
#  elif __BYTE_ORDER == __BIG_ENDIAN
    u_int16_t tcp_doff:4;
    u_int16_t tcp_res1:4;
    u_int16_t tcp_res2:2;
    u_int16_t tcp_urg:1;
    u_int16_t tcp_ack:1;
    u_int16_t tcp_psh:1;
    u_int16_t tcp_rst:1;
    u_int16_t tcp_syn:1;
    u_int16_t tcp_fin:1;
#  else
#   error "Adjust your <bits/endian.h> defines"
#  endif
    u_int16_t window;
    u_int16_t check;
    u_int16_t urg_ptr;


	char		dati[1];
};

//*************************************************************************
//  Environment Variables
    int  OnlyTest=0;
    char *dev=DEFIFACE; 
    char *ConfFile=CONFFILE; 
	char *OutRRD=RRDDatabase;
	char *PidFile=PIDFILE;
	char *Filter=NULL;
	long StepSeconds=DEFSTEP;
	char HostName[30];
	time_t start_uptime;
	time_t start_time;

	FILE	*logOther=NULL;

    bpf_u_int32 maskp;          /* subnet mask               */
    bpf_u_int32 netp;           /* ip                        */
//*************************************************************************

// function called every StepSeconds to make the update
void OutPut(void)
{
int	i, first=1, retu;
char	cmd[10240];
char	vals[10240];
char 	*line;
long 	secs_up;
char	*pc, *pv;
t_counter *cou;

pc = cmd;
pv = vals;
pc += sprintf(cmd, "rrdtool update %s -t ", OutRRD) ;
pv += sprintf(vals,"N");

for (i=0, cou = Counters; i < nCounters; i++, cou++)
	{
	if (!(cou->name))
		continue;

	if (first) first=0;
	else *pc++=':';

	pc += sprintf(pc, "in_%s:out_%s", cou->name, cou->name);
	pv += sprintf(pv, ":%ld:%ld", cou->in, cou->out);
	cou->in = cou->out = 0;
	}
 
line = malloc(strlen(cmd) + strlen(vals) +4);

sprintf(line, "%s %s", cmd, vals);

retu=system(line);

if (retu) syslog(LOG_INFO, "executed [%s] exitcode=%d; errno=%d;", line, retu, errno);

free(line);
}

//*************************************************************************

char *PrtIP(u_int32_t ip)
{
static char retu[40];
unsigned long ul;
unsigned char *x;
ul = htonl(ip);
x = (unsigned char *) &ul;

sprintf(retu, "%d.%d.%d.%d", x[3], x[2], x[1], x[0]);
return retu;
}

//*************************************************************************
// The function that analyse the traffic
u_char* handle_IP
        (u_char *args,const struct pcap_pkthdr* pkthdr,const u_char*
        packet)
{
    struct my_ip* ip;

    u_int length = pkthdr->len;
    u_int hlen,off,version;
    int i,k,j;
    int in_out;		// 1=in 0=out
    int cli_src;	// 1=cli  0=srv

    int len;

    /* jump pass the ethernet header */
    ip = (struct my_ip*)(packet + sizeof(struct ether_header));
    length -= sizeof(struct ether_header); 

    /* check to see we have a packet of valid length */
    if (length < sizeof(struct ip))
    {
        syslog(LOG_INFO, "truncated ip %d",length);
        return NULL;
    }

    len     = ntohs(ip->ip_len);
    hlen    = IP_HL(ip); /* header length */
    version = IP_V(ip);/* ip version */

    /* check version */
    if(version != 4)
    {
      syslog(LOG_INFO,"Don't ready per ip version %d",version);
      return NULL;
    }

    /* check header length */
    if(hlen < 5 )
    {
		return NULL;
    }

    /* see if we have as much packet as we should */
    if(length < len)
        syslog(LOG_INFO, "truncated IP - %d bytes missing",len - length);


	/* if source and destination are on my network and both are not me 
	 * then the triffic is not  for me
	 * */
    if (((ip->ip_dst.s_addr & maskp) == (ip->ip_src.s_addr & maskp))
	&& (ip->ip_dst.s_addr != netp )
	&& (ip->ip_src.s_addr != netp ))
	return NULL;

	//  Consider if the packet is incoming
    in_out = ((ip->ip_dst.s_addr & maskp) == (netp & maskp)) && (ip->ip_dst.s_addr != netp);


	// Where we put this bytes ?
	//
	
	switch (ip->ip_p)
		{
		case 6 	:	// TCP
				  	// Decide if we ar acting as client or server
					
				  	if (in_out)
						i = MIN(transcode_cliTCP[htons(ip->p_source)], transcode_srvTCP[htons(ip->p_dest)]);
					else
						i = MIN(transcode_srvTCP[htons(ip->p_source)], transcode_cliTCP[htons(ip->p_dest)]);
					break;
		case 17 :	// UDP
					i = MIN(transcode_UDP[htons(ip->p_source)], transcode_UDP[htons(ip->p_dest)]);
					break;
		default :
			i = idxUnClassified;
		}

	if (in_out)
		Counters[i].in += length;
	else
		Counters[i].out += length;

	// log all the establishmnent of other connection
	if (i==idxUnClassified  && logOther && ip->ip_p == 6 && (ip->tcp_ack && ip->tcp_syn)   )  
		{
		fprintf(logOther, "%ld:%s:%d <- ", time(NULL), PrtIP(ip->ip_src.s_addr), htons(ip->p_source));
		fprintf(logOther, "%s:%d |", PrtIP(ip->ip_dst.s_addr), htons(ip->p_dest), ip->ip_p);
		fputc(ip->tcp_urg ? 'U':'.',logOther);
		fputc(ip->tcp_ack ? 'A':'.',logOther);
		fputc(ip->tcp_psh ? 'P':'.',logOther);
		fputc(ip->tcp_rst ? 'R':'.',logOther);
		fputc(ip->tcp_syn ? 'S':'.',logOther);
		fputc(ip->tcp_fin ? 'F':'.',logOther);
		fputc('|', logOther);
		fputc('\n', logOther);
		fflush(logOther);
		}


return NULL;
}

/*************************************************************************************/
// Function that handles ehernet packets
u_int16_t handle_ethernet
        (u_char *args,const struct pcap_pkthdr* pkthdr,const u_char*
        packet)
{
    struct ether_header *eptr;  /* net/ethernet.h */

    /* lets start with the ether header... */
    eptr = (struct ether_header *) packet;

    /* check to see if we have an ip packet */
#ifdef DebugPrintAll
    if (ntohs (eptr->ether_type) == ETHERTYPE_IP)
    {
        fprintf(stdout,"(IP)");
    }else  if (ntohs (eptr->ether_type) == ETHERTYPE_ARP)
    {
        fprintf(stdout,"(ARP)");
    }else  if (ntohs (eptr->ether_type) == ETHERTYPE_REVARP)
    {
        fprintf(stdout,"(RARP)");
    }else {
        fprintf(stdout,"(?)");
        exit(1);
    }
    putchar('\n');
#endif

    return ntohs(eptr->ether_type);
}


/* Callback function called by the pcap library sniffer */
void my_callback(u_char *args,const struct pcap_pkthdr* pkthdr,const u_char*
        packet)
{
u_int16_t type = handle_ethernet(args,pkthdr,packet);

    switch (type)
	{
	case ETHERTYPE_IP :
	     handle_IP(args,pkthdr,packet); 
	     break;

	case ETHERTYPE_ARP :
	case ETHERTYPE_REVARP :
		 Counters[idxUnClassified].in += pkthdr->len;
	     break;
	default :
		 /**///fprintf(stdout, "Tipo indefinito(%d)\n", type);
	     break;
	}
	
}

/* Function that handles the signals */
void sighandler(int signal)
{

// Always we do an output
OutPut();
switch (signal)
    {
	case SIGALRM :
					alarm( StepSeconds - time(NULL) % StepSeconds);
					break;
    case SIGHUP  : 
    case SIGINT  : 
    case SIGQUIT : 
					syslog(LOG_INFO, "Stop signal received : Stopping daemon");
				   	exit(0);
					break;
	case SIGABRT :
	case SIGILL  :
	case SIGSEGV :
	case SIGBUS  :
					syslog(LOG_ERR, "Error signal received : Stopping daemon");
				   	exit(0);

   	break;
    }
}

//*************************************************************************

// Little Test function, for debugging
void DoTest(void)
{
typedef struct { u_int16_t a,b; } t_c;
t_c checks[] = 
{ 
	{ 80, 100, },
	{ 100, 80, },
	{ 100, 3389, },
	{ 3389, 80 },
	{ 25, 80 },
	{ 80, 25 },
	{ 1194 , 1124  },
	{ 3389 , 3389  },
	{ 1723 , 12211  },
	{ 22, 80 },
	{ 80, 22 },
	{ 10200, 4700 },
	{ 0, 0 },
};
t_c *pc;
long i;

printf("cliTCP\n");
for (i = 0; i < 65536; i++)
{
if (transcode_cliTCP[i] != idxUnClassified)
	printf("\t%5d %s\n", i, Counters[transcode_cliTCP[i]].name);
}
printf("\nsrvTCP\n");
for (i = 0; i < 65536; i++)
{
if (transcode_srvTCP[i] != idxUnClassified)
	printf("\t%5d %s\n", i, Counters[transcode_srvTCP[i]].name);
}
printf("\n-----------------------------------------------------------------\n");
		
for (pc=checks; pc->a && pc->b; pc++)
	{
	long l = MIN(transcode_cliTCP[pc->a], transcode_srvTCP[pc->b]);
	printf("src(other):%d -> dst(me):%d = %d (%s)\n", pc->a, pc->b , l, Counters[l].name);
	}

}

//*************************************************************************
void PrtUsage(char *pgname)
{
fprintf(stderr, "Usage : %s [-i:iface] [-o:output_db] [-p:pidfile] [-c:ConfFile] [-s:StepSeconds]\n", pgname);
fprintf(stderr, "\t [-r] [-f:filter]\n");
fprintf(stderr, "\n-r print on stdout parameters to create rrd_database for current configuration\n");
fprintf(stderr, "current iface      : [%s]\n", dev);
fprintf(stderr, "current ConfFile   : [%s]\n", ConfFile);
fprintf(stderr, "current rrd db     : [%s]\n", OutRRD);
fprintf(stderr, "current pidfile    : [%s]\n", PidFile);
fprintf(stderr, "current StepSeconds: [%ld]\n", StepSeconds);
}

//*************************************************************************
// Print RRD definition for current data
void PrtRRD(void)
{
int i;
char lbl[40];
long	l;

t_counter *cou;


l = time(NULL);
l -= l % StepSeconds;
printf("rrdtool create %s \\\n", OutRRD);
printf("--start %ld \\\n", l);
printf("--step %ld \\\n", StepSeconds);

for (i=0, cou = Counters; i < nCounters; i++, cou++)
	{
	if (!(cou->name))
		continue;

	printf("DS:in_%s:ABSOLUTE:%ld:0:U \\\n", cou->name, StepSeconds * 2);
	printf("DS:out_%s:ABSOLUTE:%ld:0:U \\\n", cou->name, StepSeconds * 2);
	}

if (StepSeconds >= 60)
	{
	// 40 ore dettaglio Step
	printf("RRA:LAST:0.5:%ld:%ld \\\n", 1, (40 * 60 * 60)/StepSeconds  );
	}

if (StepSeconds <= 1800)
	{
	// Una settimana, dettaglio mezz'ora
	printf("RRA:MAX:0.5:%ld:%ld \\\n", 1800 / StepSeconds, (24 * 2) * 8);
	printf("RRA:AVERAGE:0.5:%ld:%ld \\\n", 1800 / StepSeconds, (24 * 2) * 8 );
	}

// Un mese, dettaglio 1 ore
printf("RRA:MAX:0.5:%ld:%ld \\\n", 3600 / StepSeconds, (24 * 40));
printf("RRA:AVERAGE:0.5:%ld:%ld \\\n", 3600 / StepSeconds, (24 * 40));

// Un anno e mezzo, dettaglio 1 giorno
printf("RRA:MAX:0.5:%ld:%ld \\\n", 86400 / StepSeconds, 480);
printf("RRA:AVERAGE:0.5:%ld:%ld \n", 86400 / StepSeconds, 480);
}

//*************************************************************************

void LoadHostname(void)
{
FILE	*fl;
char *p;
fl = popen("/bin/hostname", "r");
fgets(HostName, sizeof(HostName), fl);
pclose(fl);

HostName[sizeof(HostName)-1] = '\0';
if (p=strchr(HostName, '\n')) *p='\0';
}

//*************************************************************************

void Load_UpTime(void)
{
FILE	*fl;
char	line[1024];
fl = fopen("/proc/uptime", "r");
fgets(line, sizeof(line), fl);
fclose(fl);

start_uptime = atol(line);
time(&start_time);
}

// Functions for the interpretation of configuration file
//*************************************************************************
// Retrive
long GetCounterIdx(char *CounterName)
{
long	retu;

for (retu = 0; retu < nCounters; retu++)
	{
	if	(!strcmp(CounterName, Counters[retu].name))
		return retu;
	}
return -1;
}

//*************************************************************************
void	AddPortRange(char *CounterName, int tcp_udp_all, unsigned short fromPort, unsigned short toPort)
{
int CounterIdx = GetCounterIdx(CounterName);
long	i;

if (CounterIdx < 0) 
	{
	fprintf(stderr, "Error adding ports to [%s] : Counter not found\n", CounterName);
	return;
	}

if (fromPort == 0) fromPort=1;

for (i = fromPort ; i <= toPort; i++)
	{
	if (tcp_udp_all & TRAF_TCP)
		{
		if ((tcp_udp_all & TRAF_CLI) && transcode_cliTCP[i] == NOCOUNTER)
				transcode_cliTCP[i] = CounterIdx;
		if ((tcp_udp_all & TRAF_SRV) && transcode_srvTCP[i] == NOCOUNTER)
				transcode_srvTCP[i] = CounterIdx;
		}
	if (tcp_udp_all & TRAF_UDP)
		{
		if (transcode_UDP[i] == NOCOUNTER)
			transcode_UDP[i] = CounterIdx;
		}
	}
}

//*************************************************************************

void SplitStringParts(char **StringParts, char *str)
{
int part=0;
char	*p;

if (p = strchr(str, '#' )) *p='\0';
if (p = strchr(str, '\r')) *p='\0';
if (p = strchr(str, '\n')) *p='\0';
p = str;
while (part < 1000 && *p)
	{
	StringParts[part++] = p;

	while (!isspace(*p) && *p) p++;
	if (*p) *p++ = '\0';
	while (isspace(*p) && *p) p++;
	}
StringParts[part++] = NULL;
}

//*************************************************************************

int LoadParam(int argc, char *argv[])
{
int 	i, ParamErr=0;
FILE	*fl;
char	line[1024];
char	*StringParts[1000];

// Leggo parametri
for (i=1; i < argc ; i++)
	{
	if (argv[i][0] == '-' && argv[i][2] == ':')
		switch (argv[i][1])
			{
			case 'i' : dev = argv[i]+3;
					   break;
			case 'o' : OutRRD = argv[i]+3;
					   break;
			case 's' : StepSeconds = atol(argv[i]+3);
					   if (StepSeconds < 10 || StepSeconds > 3600)
						   ParamErr = 1;
					   break;
			case 'p' : PidFile = argv[i]+3;
					   break;
			case 'c' : ConfFile = argv[i]+3;
					   break;
			case 'f' : Filter = argv[i]+3;
					   break;
			default :
					  ParamErr = 1;
			}
	else if (!strcmp(argv[i] , "-r") && !ParamErr)
		ParamErr = -1;
	else if (!strcmp(argv[i] ,"-t") && !ParamErr)  
		OnlyTest = 1;
	else
		ParamErr = 1;
	}

if (OutRRD == RRDDatabase)
	{
	OutRRD = malloc(256);
	sprintf(OutRRD, RRDDatabase, dev);
	}

if (PidFile == PIDFILE)
	{
	PidFile = malloc(256);
	sprintf(PidFile, PIDFILE, dev);
	}

if (ParamErr <= 0)
	{
	for(i=0; i < sizeof(transcode_UDP) /sizeof(*transcode_UDP); i++)
		{
		transcode_cliTCP[i] = NOCOUNTER;
		transcode_srvTCP[i] = NOCOUNTER;
		transcode_UDP[i] = NOCOUNTER;
		}

	aCounters=10;
	Counters = malloc(sizeof(*Counters) * aCounters);
	// open ConfFile
	fl=fopen(ConfFile, "r");
	if (!fl)
		{
		perror("Opening configuration file");
		ParamErr = 1;
		}
	else
		{
		// Load all rous of type COUNTER
		while (fgets(line, sizeof(line), fl))
			{
			SplitStringParts(StringParts, line);

			if (StringParts[0] == NULL) continue;

			if (!strcmp(StringParts[0], "COUNTER"))
				{
				if (nCounters+1 >= aCounters)
					{
					aCounters += 10;
					if (Counters)
						Counters = realloc(Counters, sizeof(*Counters) * aCounters);
					}
				Counters[nCounters].name = malloc(strlen(StringParts[1]));
				strcpy(Counters[nCounters].name, StringParts[1]);
				nCounters++;
				}
			}
		// Add a counter as UnClassified
		idxUnClassified = nCounters++;
		Counters[idxUnClassified].name = "UnClassified";
		rewind(fl);


		// Now loads tha port and portrange attributions
		while (fgets(line, sizeof(line), fl))
			{
			int	tcp_udp_all=0;
			char	*p;
			SplitStringParts(StringParts, line);
			if (StringParts[0] == NULL) continue;

			if (!strncmp(StringParts[0],"ALL:",4))
				tcp_udp_all=TRAF_TCP | TRAF_UDP | TRAF_SRV | TRAF_CLI;
			else if (!strncmp(StringParts[0],"TCP:",4))
				tcp_udp_all=TRAF_TCP;
			else if (!strncmp(StringParts[0],"UDP:",4))
				tcp_udp_all=TRAF_UDP;

			if (tcp_udp_all & 1)
				{
				p=strchr(StringParts[0]+4, ':');
				if (p) *p++='\0';
				if (p && !strcmp(p, "SRV"))
					tcp_udp_all = TRAF_TCP | TRAF_SRV;
				if (p && !strcmp(p, "CLI"))
					tcp_udp_all = TRAF_TCP | TRAF_CLI;
				if (p && !strcmp(p, "ALL"))
					tcp_udp_all = TRAF_TCP | TRAF_CLI | TRAF_SRV;
				}

			if (!strcmp(StringParts[0]+4, "PORTRANGE"))
				{
				AddPortRange(StringParts[1], tcp_udp_all, atol(StringParts[2]), atol(StringParts[3]));
				}
			if (!strcmp(StringParts[0]+4, "PORT"))
				{
				int i;
				long	port;

				for (i = 2; i< (sizeof(StringParts)/sizeof(*StringParts)) && StringParts[i]; i++)
					{
					struct servent *ps;
					if (!isdigit(*StringParts[i]))
						{
						ps=getservbyname(StringParts[i], (tcp_udp_all & TRAF_TCP) ? "tcp" :"udp");
						if (!ps) fprintf(stderr, "Parsing :not recognized [%s]\n", StringParts[i]);
						}
					else
						ps = NULL;
					if (ps)
						{
						AddPortRange(StringParts[1],tcp_udp_all, htons(ps->s_port), htons(ps->s_port));
						}
					else
						{
						AddPortRange(StringParts[1],tcp_udp_all, atol(StringParts[i]), atol(StringParts[i]));
						}
					}
				}
			}
		fclose(fl);

		// now set all the ports unassigned to unclassified
		for (i=0; i< 65536; i++)
			{
			if (transcode_cliTCP[i] == NOCOUNTER) transcode_cliTCP[i] = idxUnClassified;
			if (transcode_srvTCP[i] == NOCOUNTER) transcode_srvTCP[i] = idxUnClassified;

			if (transcode_UDP[i] == NOCOUNTER) transcode_UDP[i] = idxUnClassified;
			}
		}
	}

return ParamErr;
}

//*************************************************************************
int main(int argc,char **argv)
{ 
    int i;
    char errbuf[PCAP_ERRBUF_SIZE];
    pcap_t* descr;
    const u_char *packet;
    struct pcap_pkthdr hdr;     /* pcap.h                    */
    struct ether_header *eptr;  /* net/ethernet.h            */
    struct bpf_program fp;      /* hold compiled program     */
	pid_t pid, sid;
	int	ParamErr=0;


#ifdef Filter
    if(argc != 2){ fprintf(stdout,"Usage: %s \"filter program\"\n"
            ,argv[0]);return 0;}
#endif
	
	ParamErr = LoadParam(argc, argv);

	if (ParamErr >0)
		{
		PrtUsage(argv[0]);
		return 1;
		}

	if (OnlyTest)
		{
		DoTest();
		return 0;
		}

	// Initialization
	LoadHostname();
	Load_UpTime();

	if (ParamErr < 0)
		{
		PrtRRD();
		return 0;
		}

	//daemonize me
	

	// For debugging
#ifndef NODAEMON
	pid=fork();
	if (pid < 0)
		{
		fprintf(stderr, "Impossible to fork()\n");
		return 1;
		}

	umask(0);
	if (pid > 0)
		{
		// This is the parent, so write the pid
		FILE *fl;
		if (fl=fopen(PidFile, "w"))
			{
			fprintf(fl, "%ld\n", pid);
			fclose(fl);
			}
		else
			{
			fprintf(stderr, "Impossible to write pid file %s\n", PidFile);
			}
		return 0;
		}
	// ************************************************************************
	// Here are the daemon code

	// ... a cut with the past 
	close(STDIN_FILENO);
	close(STDOUT_FILENO);
	close(STDERR_FILENO);

	// Open the log
	openlog(LOGID, LOG_PID, LOG_DAEMON);
	logOther=fopen("/var/log/mplt_other", "a");
	fprintf(logOther, "%ld:mpltd started on %s\n", time(NULL), dev);
	fflush(logOther);

	sid=setsid();
	if (sid < 0) 
		{
	    /* Log the failure */
		syslog(LOG_ERR, "Impossible to setsid()");
		closelog();
	    exit(EXIT_FAILURE);
	    }
	

	// Signal have to be catched
    signal(SIGALRM , sighandler);
    signal(SIGQUIT , sighandler); 
    signal(SIGINT  , sighandler); 
	signal(SIGABRT , sighandler);
   /* signal(SIGHUP  , sighandler); 
	signal(SIGILL  , sighandler);
	signal(SIGSEGV , sighandler);
	signal(SIGBUS  , sighandler);*/

	alarm( StepSeconds - time(NULL) % StepSeconds);

#endif

    /* ask to pcap for the network address and mask of the device */
    if (pcap_lookupnet(dev,&netp,&maskp,errbuf))
		{
		/* grab a device to peak into... */
		dev = pcap_lookupdev(errbuf);
		if(dev == NULL)
			{ 
			syslog(LOG_ERR, errbuf);
			closelog();
			exit(1); 
			}
    	pcap_lookupnet(dev,&netp,&maskp,errbuf);
		}
	syslog(LOG_INFO, "Daemon started pid %ld sid %ld dev %s IP %08lx Mask %08lx", getpid(), sid, dev, netp, maskp );
	


    /* open device for reading */
    descr = pcap_open_live(dev,BUFSIZ,0,0,errbuf);

    if(descr == NULL)
		{
		syslog(LOG_ERR, "pcap_open_live(): %s",errbuf);
		closelog();
		exit(1);
		}

	if (Filter)
		{
		if(pcap_compile(descr,&fp,Filter,0,netp) == -1)
			{ 
			syslog(LOG_ERR,"Error calling pcap_compile on filter [%s] pcap:[%s]", Filter, pcap_geterr(descr)); 
			closelog();
			exit(1); 
			}

		/* set the compiled program as the filter */
		if(pcap_setfilter(descr,&fp) == -1)
			{ 
			syslog(LOG_ERR,"Error setting filter pcap:[%s]", pcap_geterr(descr)); 
			closelog();
			exit(1); 
			}
		}

    /* ... and loop */ 
    pcap_loop(descr,-1,my_callback,NULL);

	syslog(LOG_INFO, "Stopping daemon");
	closelog();
    return 0;
}
